Package: GoodFibes
Type: Package
Title: Detection and Reconstruction of Muscle Fibers from diceCT Image
        Data
Version: 1.0.0
Date: 2026-04-21
Authors@R: person(given = "Jessica",
                      family = "Arbour",
                      role = c("aut", "cre"),
                      email = "jessica.arbour@mtsu.edu")
Maintainer: Jessica Arbour <jessica.arbour@mtsu.edu>
Description: Reconstruction of muscle fibers from image stacks using textural analysis. Includes functions for tracking, smoothing, cleaning, plotting and exporting muscle fibers. Also calculates basic fiber properties (e.g., length, angle and curvature).
License: GPL (>= 2)
Depends: R(>= 3.5.0)
Imports: rgl, stats, graphics, grDevices, concaveman, prodlim,
        splines2, imager, matlib
NeedsCompilation: no
Packaged: 2026-04-21 22:44:03 UTC; fount
Author: Jessica Arbour [aut, cre]
Repository: CRAN
Date/Publication: 2026-04-23 15:00:02 UTC
Built: R 4.6.0; ; 2026-05-01 01:47:38 UTC; unix

Package: GoodFibes
Type: Package
Title: Detection and Reconstruction of Muscle Fibers from diceCT Image
        Data
Version: 1.0.0
Date: 2026-04-21
Authors@R: person(given = "Jessica",
                      family = "Arbour",
                      role = c("aut", "cre"),
                      email = "jessica.arbour@mtsu.edu")
Maintainer: Jessica Arbour <jessica.arbour@mtsu.edu>
Description: Reconstruction of muscle fibers from image stacks using textural analysis. Includes functions for tracking, smoothing, cleaning, plotting and exporting muscle fibers. Also calculates basic fiber properties (e.g., length, angle and curvature).
License: GPL (>= 2)
Depends: R(>= 3.5.0)
Imports: rgl, stats, graphics, grDevices, concaveman, prodlim,
        splines2, imager, matlib
NeedsCompilation: no
Packaged: 2026-04-21 22:44:03 UTC; fount
Author: Jessica Arbour [aut, cre]
Repository: CRAN
Date/Publication: 2026-04-23 15:00:02 UTC
Built: R 4.4.3; ; 2026-04-23 16:48:17 UTC; unix

export("good.fibes", "fibers.smoothed", "muscle.plot", "muscle.plot.multi",
       "fiber.lengths", "crop.stack", "thresholdPlot", "sequencePlot",
       "equalize.stack", "quality.check", "check.overlap", "muscle.plot.stl",
       "color.scale", "fuse.fibers", "fuse.fibers.auto", "fiber.curve", "fiber.angle","find.endpoints")
importFrom("rgl", plot3d, points3d, lines3d, open3d, writeSTL,cylinder3d,writePLY,shade3d,wire3d)
importFrom("concaveman",concaveman)
importFrom("grDevices",colorRampPalette, boxplot.stats)
importFrom("prodlim",row.match)
importFrom("splines2",nsp)
importFrom("stats",lm, predict,hclust,cutree,sd,quantile,IQR,dist,ecdf,na.omit,kmeans,prcomp)
importFrom("graphics",plot,points,barplot,text,abline)
import("imager")
importFrom("matlib",angle)

equalize.stack <-
function(images, n, save.images=FALSE){
 
  slice<-load.image(images[[n]])
  
  eqf <- ecdf(slice)
  eqslice<-as.cimg(eqf(slice),dim=dim(slice))
  eqslice<-eqslice-min(eqslice)
  eqslice<-eqslice/max(eqslice)
  plot(eqslice)
  
  if(save.images){
  for(i in 1:length(images)){
    slice<-load.image(images[[i]])
    greys<-slice[which(slice>0)]
    
    if(length(greys)==0){save.image(slice,file=paste("eq_",images[i],".png",sep=""))
    } else {
      
      eqf <- ecdf(slice)
      eqslice<-as.cimg(eqf(slice),dim=dim(slice))
      eqslice<-eqslice-min(eqslice)
      eqslice<-eqslice/max(eqslice)
      save.image(eqslice, file=paste("eq_",images[i],".png",sep=""))
    }
  }
  }
 
}

fuse.fibers <-
function(fiber.list,min.vox,min.improvement=0.25,df=2,length.out=100){
  fl<-fiber.lengths(fiber.list,res=1,df=df)
  combine.mat<-c(NA,NA)
  fused.list<-vector(mode="list")
  
  for(i in 1:length(fiber.list)){
    for(j in 1:length(fiber.list)){
      if(i==j){next()}
      if(i %in% combine.mat){next()}
      if(j %in% combine.mat){next()}
      
      fb.df1<-data.frame(x = fiber.list[[i]]$fiber.points[,1],y=fiber.list[[i]]$fiber.points[,2],z=fiber.list[[i]]$fiber.points[,3])
      newdata<-seq(min(fb.df1$z),max(fb.df1$z),length.out=length.out)
      if(df==1){ fit1 <- tryCatch(lm(cbind(x,y)~z, data = fb.df1))
      } else {
        fit1 <- tryCatch(lm(cbind(x,y)~nsp(z,df=df), data = fb.df1))}
      pred1 <- cbind(predict(fit1, newdata = list(z = newdata)))
      
      fb.df2<-data.frame(x = fiber.list[[j]]$fiber.points[,1],y=fiber.list[[j]]$fiber.points[,2],z=fiber.list[[j]]$fiber.points[,3])
      newdata<-seq(min(fb.df2$z),max(fb.df2$z),length.out=length.out)
      if(df==1){ fit2 <- tryCatch(lm(cbind(x,y)~z, data = fb.df2))
      } else {
        fit2 <- tryCatch(lm(cbind(x,y)~nsp(z,df=df), data = fb.df2))}
      pred2 <- cbind(predict(fit2, newdata = list(z = newdata)))
      
      
      two.fibs.list<-rbind(pred1,pred2)
      dist.mat<-as.matrix(dist(two.fibs.list))
      
      min.dists<-apply(dist.mat[1:length.out,(length.out+1):(length.out*2)],1,min)
      is.close<-sum(ifelse(min.dists<min.vox,1,0))
      if(is.close==0){next()}
      
      combined.fiber<-rbind(fiber.list[[i]]$fiber.points,fiber.list[[j]]$fiber.points)
      order.ref<-order(combined.fiber[,3])
      combined.fiber<-combined.fiber[order.ref,]
      combined.list<-list(fiber.points=combined.fiber)
      comb.fl<-fiber.lengths(combined.list)
      improvement<-(comb.fl/max(fl[c(i,j)]))-1
      
      if(improvement < min.improvement){next()}
      
      new.df<-data.frame(x=combined.fiber[,1],y=combined.fiber[,2],z=combined.fiber[,3])
      if(df==1){ fit <- tryCatch(lm(cbind(x,y)~z, data = new.df))
      } else {
        fit <- tryCatch(lm(cbind(x,y)~nsp(z,df=df), data = new.df))}
      fib.smoothed<-cbind(predict(fit, newdata=list(z=combined.fiber[,3])), combined.fiber[,3])
      
      resid1<-sqrt(apply(((predict(fit, newdata=list(z=fiber.list[[i]]$fiber.points[,3])))-
                            fiber.list[[i]]$fiber.points[,1:2])^2,1,sum))
      resid2<-sqrt(apply(((predict(fit, newdata=list(z=fiber.list[[j]]$fiber.points[,3])))-
                            fiber.list[[j]]$fiber.points[,1:2])^2,1,sum))
      resid<-c(resid1,resid2)
      
      resid.original<-sqrt(apply(rbind(fit1$residuals,fit2$residuals)^2,1,sum))
      
      if(mean(resid)<min.vox){
        combine.mat<-rbind(combine.mat,c(i,j))
        fiber.points<-combined.fiber
        fused.list[[dim(combine.mat)[1]]]<-fiber.points
      } else {next()}
      
    } 
  }
  
  if(is.null(dim(combine.mat))){message("there are no fibers to fuse\n")
    fused<-0
    return(fused)
  } else {
    
    
    combine.mat<-combine.mat[2:nrow(combine.mat),]
    fused.list<-fused.list[2:length(fused.list)]
    if(!is.matrix(combine.mat)){
      combine.mat<-matrix(combine.mat,nrow=1,ncol=2)
    }
    
    colnames(combine.mat)<-c("fiber1","fiber2")
    new.fiber.list<-fiber.list
    for(i in 1:length(fused.list)){
      fiber.points<-as.matrix(fused.list[[i]])
      new.fiber.list[[combine.mat[i,1]]]$fiber.points<-fiber.points
    }
    new.fiber.list<-new.fiber.list[-combine.mat[,2]]
    return(list(fibers.to.merge=combine.mat,merged.fibers=new.fiber.list))
    
  }
}

muscle.plot <-
function(fiber.dat,images,df=4,mirror.axis=FALSE,outline=50, size=2){
  if(mirror.axis){fiber.dat[,2]<-fiber.dat[,2]*-1}
  open3d()
  
  sliceseq<-round(seq(1,length(images),length.out=outline))
  boundaries<-vector(length = length(images),mode="list")
  for(i in sliceseq){
    slice<-which(!load.image(images[i])==0, arr.ind = T)[,1:2]  
    if(!is.matrix(slice)){next()}
    if(dim(slice)[1]==0){next()}
    boundaries[[which(sliceseq==i)]]<-concaveman(slice)
  }

   for(k in 1:length(sliceseq)){
    if(is.null(boundaries[[k]])){next()}
    bound<-cbind(boundaries[[k]],rep(sliceseq[k],nrow(boundaries[[k]])))
    
    if(mirror.axis){
      points3d(bound[,1],bound[,2]*-1,bound[,3],alpha=0.02)
      } else {
      points3d(bound[,1],bound[,2],bound[,3],alpha=0.02)
      }
   }
  points3d(fiber.dat,col="red",size=size)
  fb.df <- data.frame(x = fiber.dat[, 1], y = fiber.dat[,2], z = fiber.dat[, 3])
  if(df==1){ fit <- tryCatch(lm(cbind(x,y)~z, data = fb.df))
  } else {
    fit <- tryCatch(lm(cbind(x,y)~nsp(z,df=df), data = fb.df))}
  lines3d(cbind(predict(fit, newdata = list(z = fiber.dat[,3])), fiber.dat[,3]))
}

good.fibes <- function(images, zero.image, radius, threshold=NULL, cutoff, scaler=1, blackcut=0.95, seeds=1, show.plot=TRUE,start.seed=NULL,allowed.black=0,bound.buffer=0,backstep=0,verbose=TRUE){
  depth<-radius+1
  if(length(channels(load.image(images[1])))>1){stop("images must be grayscale, try converting with imageJ (toggle option under import image sequence then resave")}
  if(is.null(threshold)){threshold<-cutoff}
  if(radius > 11){stop("radius cannot be more than 11")}
  if(threshold < cutoff){stop("threshold must be equal to or greater than cutoff")}
  
  res.list<-vector(length=seeds,mode="list")
  
  #### starting seed point
  abovethres<-as.matrix(which(load.image(images[zero.image])>threshold,arr.ind = T))[,1:2]
  
  if(verbose){cat("choosing starting points","\n",sep=" ")}
  
  thres.dist<-dist(abovethres) 
  thresclust<-hclust(thres.dist, method="average")  
  thres.cut<-cutree(thresclust,k=seeds)
  
  loc<-matrix(ncol=2,nrow=seeds)
  for(tc in 1:seeds){
    grouptc<-which(thres.cut==tc)
    if(!is.null(start.seed)){set.seed(start.seed)}
    loc[tc,]<-abovethres[sample(grouptc,1),]
  }
  
  
  
  for(zz in 1:seeds){
    tryCatch({
      if(seeds > 1){
        startx<-loc[zz,1]
        starty<-loc[zz,2]
      } else {
        startx<-loc[1]
        starty<-loc[2]
      }
      
      ######
      
      if(show.plot){
        plot(load.image(images[zero.image]),main = "foward walk")
        points(startx,starty,col="red",pch=16,cex=1)
      }
      
      ##### creating image list
      imseq<-zero.image:(zero.image+radius)
      imglist<-vector(length=depth,mode="list")
      for(i in imseq){
        imglist[[which(imseq==i)]]<-load.image(file=images[i])
      }
      for(j in 1:depth){
        imglist[[j]][which(imglist[[j]]<cutoff)]<-0
      }
      
      #### creating the first data matrix
      tracker<-1
      fib.track<-matrix(ncol=3,nrow=length(images))
      ## record starting point
      fib.track[1,]<-c(startx,starty,zero.image)
      
      
      
      ############ launch ##############
      
      
      
      
      
      
      ########################################################
      ################## Forward #############################
      ########################################################
      if(verbose){cat("beginning forward walk from zero image for fiber",zz,"\n",sep=" ")
      }
      ### first measurements
      
      ucoords<-hemisphere.points(radius)
      lpl<-pointsGenerator(startx = startx, starty = starty, ucoords=ucoords, radius=radius,backstep=backstep)
      
      white.mat<-get.whites(imglist,lpl,depth)
      white.sd<-apply(white.mat,1,sd)
      white.sd<-white.sd/max(white.sd)
      
      ### no initial trajectory, all paths equally probable
      dist.check<-rep(1,length=nrow(white.mat))
      
      white.optim<-white.sd * dist.check^scaler
      white.optim<-white.optim/max(white.optim)
      cross.check<-apply(ifelse(white.mat[,2:radius]==0,1,0),1,sum)
      cross.check<-ifelse(cross.check<=allowed.black,0,1000)
      end.check<-ifelse(white.mat[,depth]==0,1000,0)
      white.optim<-white.optim + cross.check + end.check
      
      #### get dist check for next step before moving on
      
      
      
      ####### first step
      best<-which(white.optim==min(white.optim))
      if(length(best)>1){sample(best,1)}
      
      
      new.dat<-lpl[[best]][nrow(lpl[[best]]),]
      traj.ref<-lpl[[best]][nrow(lpl[[best]]),]-c(startx,starty,0)
      new.startx<-new.dat[1]
      new.starty<-new.dat[2]
      new.zero<-new.dat[3]
      new.zero.image<-zero.image+new.zero
      
      
      
      if(show.plot){
        plot(load.image(images[new.zero.image]),main="forward walk")
        points(new.startx,new.starty,col="red",pch=16,cex=1)
      }
      
      
      
      tracker<-tracker+1
      fib.track[tracker,]<-c(new.startx,new.starty,new.zero.image)
      
      
      
      ######## repeater for steps 3 and onwards
      
      for(z in 1:length(images)){
        
        ucoords<-hemisphere.points(radius,backstep=backstep)
        lpl<-pointsGenerator(startx = new.startx, starty = new.starty, ucoords = ucoords, radius = radius,backstep=backstep)
        
        
        imseq<-(new.zero.image-backstep):(new.zero.image+radius)
        for(i in imseq){
          imglist[[which(imseq==i)]]<-load.image(file=images[i])
        }
        for(j in 1:length(imglist)){
          imglist[[j]][which(imglist[[j]]<cutoff)]<-0
        }
        
        white.mat<-get.whites(imglist,lpl,depth,backstep=backstep)
        
        str.dist<-vector(length=nrow(white.mat))
        for(q in 1:length(str.dist)){
          other.coords<- lpl[[q]][nrow(lpl[[q]]),]-c(new.startx,new.starty,0)
          str.dist[q]<-sqrt(sum((traj.ref-other.coords)^2))
        }
        
        dist.check<-str.dist/max(str.dist)+1
        white.sd<-apply(white.mat,1,sd)
        white.sd<-white.sd/max(white.sd)
        
        white.optim<-white.sd * dist.check^scaler
        white.optim<-white.optim/max(white.optim)
        
        cross.check<-apply(ifelse(white.mat[,2:radius]==0,1,0),1,sum)
        cross.check<-ifelse(cross.check<=allowed.black,0,1000)
        end.check<-ifelse(white.mat[,depth]==0,1000,0)
        white.optim<-white.optim + cross.check + end.check
        
        best<-which(white.optim==min(white.optim))
        if(length(best)>1){sample(best,1)}
        
        rep.check<-row.match(c(0,0,new.zero.image)+lpl[[best]][nrow(lpl[[best]]),], as.data.frame(fib.track), nomatch=0)
        if(rep.check>0){best<-order(white.optim)[2]}
        
        
        
        if(sum(ifelse(white.optim>=1000,0,1))==0){break()}
        
        
        ##check if the end points of the possible lines are black (outside muscle)
        ends.black<-which(white.mat[,depth]==0)
        ends.black<-length(ends.black)/nrow(white.mat)
        
        if(ends.black>blackcut){break()} 
        
        new.dat<-lpl[[best]][nrow(lpl[[best]]),]
        traj.ref<-lpl[[best]][nrow(lpl[[best]]),]-c(new.startx,new.starty,0)
        new.startx<-new.dat[1]
        new.starty<-new.dat[2]
        new.zero<-new.dat[3]
        new.zero.image<-new.zero.image+new.zero
        
        
        
        if(new.zero.image>(length(images)-depth)){break()}
        
        
        if(boundary.check(images, new.zero.image, new.startx, new.starty, bound.buffer, cutoff)>0){break()} 
        
        
        
        tracker<-tracker+1
        fib.track[tracker,]<-c(new.startx,new.starty,new.zero.image)
        
        
        
        if(show.plot){
          plot(load.image(images[new.zero.image]),main="forward walk")
          points(new.startx,new.starty,col="red",pch=16,cex=1)
        }
      }
      
      
      
      
      
      
      ##########################################
      ################# backward ###############
      ##########################################
      if(verbose){cat("beginning backward walk from zero image for fiber",zz,"\n",sep=" ")
      }
      ###reset
      ucoords<-hemisphere.points(radius)
      lpl<-pointsGenerator(startx = startx, starty = starty, ucoords = ucoords, radius = radius,backstep=backstep)
      
      
      
      imseq<-zero.image:(zero.image-radius)
      imglist<-vector(length=depth,mode="list")
      for(i in imseq){
        imglist[[which(imseq==i)]]<-load.image(file=images[i])
      }
      for(j in 1:depth){
        imglist[[j]][which(imglist[[j]]<cutoff)]<-0
      }
      
      
      white.mat<-get.whites(imglist,lpl,depth,backstep=0)
      white.sd<-apply(white.mat,1,sd)
      white.sd<-white.sd/max(white.sd)
      
      
      
      dist.check<-rep(1,length=nrow(white.mat))
      
      
      
      tracker<-1
      fib.trackback<-matrix(ncol=3,nrow=length(images))
      ## record starting point
      fib.trackback[1,]<-c(startx,starty,zero.image)
      
      ############ launch ##############
      
      
      
      
      white.optim<-white.sd * dist.check^scaler 
      white.optim<-white.optim/max(white.optim)
      
      cross.check<-apply(ifelse(white.mat[,2:radius]==0,1,0),1,sum)
      cross.check<-ifelse(cross.check<=allowed.black,0,1000)
      end.check<-ifelse(white.mat[,depth]==0,1000,0)
      white.optim<-white.optim + cross.check + end.check
      
      
      
      ####### first step
      best<-which(white.optim==min(white.optim))
      if(length(best)>1){best<-sample(best,1)}
      
      
      
      rep.check<-row.match(lpl[[best]][nrow(lpl[[best]]),], as.data.frame(fib.trackback), nomatch=0)
      if(rep.check>0){best<-order(white.optim)[2]}
      
      
      
      new.dat<-lpl[[best]][nrow(lpl[[best]]),]
      traj.ref<-lpl[[best]][nrow(lpl[[best]]),]-c(startx,starty,0)
      new.startx<-new.dat[1]
      new.starty<-new.dat[2]
      new.zero<-new.dat[3]
      new.zero.image<-zero.image-new.zero
      
      if(show.plot){
        plot(load.image(images[new.zero.image]),main="backward walk")
        points(new.startx,new.starty,col="red",pch=16,cex=1)
      }
      
      
      
      tracker<-tracker+1
      fib.trackback[tracker,]<-c(new.startx,new.starty,new.zero.image)
      
      ##### repeater for steps 3 and onward
      
      for(z in 1:length(images)){
        
        
        
        ucoords<-hemisphere.points(radius,backstep=backstep)
        lpl<-pointsGenerator(startx = new.startx, starty = new.starty, ucoords = ucoords, radius = radius,backstep=backstep)
        
        
        ### new image list
        imseq<-(new.zero.image+backstep):(new.zero.image-radius)
        for(i in imseq){
          imglist[[which(imseq==i)]]<-load.image(file=images[i])
        }
        for(j in 1:depth){
          imglist[[j]][which(imglist[[j]]<cutoff)]<-0
        }
        
        
        
        white.mat<-get.whites(imglist,lpl,depth,backstep=backstep)
        
        str.dist<-vector(length=nrow(white.mat))
        for(q in 1:length(str.dist)){
          other.coords<- lpl[[q]][nrow(lpl[[q]]),]-c(new.startx,new.starty,0)
          str.dist[q]<-sqrt(sum((traj.ref-other.coords)^2))
        }
        
        
        dist.check<-str.dist/max(str.dist)+1
        
        
        
        white.sd<-apply(white.mat,1,sd)
        white.sd<-white.sd/max(white.sd)
        
        white.optim<-white.sd * dist.check^scaler
        white.optim<-white.optim/max(white.optim)
        
        cross.check<-apply(ifelse(white.mat[,2:radius]==0,1,0),1,sum)
        cross.check<-ifelse(cross.check<=allowed.black,0,1000)
        end.check<-ifelse(white.mat[,depth]==0,1000,0)
        white.optim<-white.optim + cross.check + end.check
        
        if(sum(ifelse(white.optim>=1000,0,1))==0){break()}
        
        
        ##check if the end points of the possible lines are black (outside muscle)
        ends.black<-which(white.mat[,depth]==0)
        ends.black<-length(ends.black)/nrow(white.mat)
        
        if(ends.black>blackcut){break()} 
        
        best<-which(white.optim==min(white.optim))
        if(length(best)>1){best<-sample(best,1)}
        
        rep.check<-row.match(c(0,0,new.zero.image)+lpl[[best]][nrow(lpl[[best]]),]*(c(1,1,-1)), as.data.frame(fib.trackback), nomatch=0)
        if(rep.check>0){best<-order(white.optim)[2]}
        
        
        new.dat<-lpl[[best]][nrow(lpl[[best]]),]
        traj.ref<-lpl[[best]][nrow(lpl[[best]]),]-c(new.startx,new.starty,0)
        new.startx<-new.dat[1]
        new.starty<-new.dat[2]
        new.zero<-new.dat[3]
        new.zero.image<-new.zero.image-new.zero
        if(new.zero.image<depth){break()}
        
        imseq<-new.zero.image:(new.zero.image-radius)
        for(i in imseq){
          imglist[[which(imseq==i)]]<-load.image(file=images[i])
        }
        for(j in 1:depth){
          imglist[[j]][which(imglist[[j]]<cutoff)]<-0
        }
        
        tracker<-tracker+1
        fib.trackback[tracker,]<-c(new.startx,new.starty,new.zero.image)
        
        if(show.plot){
          plot(load.image(images[new.zero.image]),main="backward walk")
          points(new.startx,new.starty,col="red",pch=16,cex=1)
        }
        
        if(boundary.check(images, new.zero.image, new.startx, new.starty, bound.buffer, cutoff)>0){
          break()
        } 
      }
      
      
      fib.track<-fib.track[!is.na(fib.track[,1]),]
      fib.trackback<-fib.trackback[!is.na(fib.trackback[,1]),]
      
      ntb<-nrow(fib.trackback)
      
      fib.final<-rbind(fib.trackback[ntb:1,], fib.track[2:nrow(fib.track),])
      
      res.list[[zz]]<-list(fiber.points=fib.final)
      
    }, error = function(msg){res.list[[zz]]<-list(fiber.points=NA)})
    
    
  }
  
  if(seeds == 1){
    res.list<-res.list[[1]]
  }
  drop.check<-sapply(res.list,function(x){length(x$fiber.points)}) 
  keeps<-which(drop.check>1)
  if(length(keeps)<seeds & verbose){cat(seeds-length(keeps),"fiber did not run properly, dropped\n",sep=" ")}
  res.list<-res.list[keeps]
  
  return(res.list)
  
}

get.whites <-
function(imglist,lpl,depth,backstep=0){
  white.mat<-matrix(nrow=length(lpl),ncol=depth)
  white<-vector(length=depth)
  for(m in 1:length(lpl)){
    one.line<-lpl[[m]]
    white[1]<-as.matrix(imglist[[1]])[one.line[1,1],one.line[1,2]]
    white[2]<-as.matrix(imglist[[one.line[2,3]+1]])[one.line[2,1],one.line[2,2]]
    if(depth>=3){
      white[3]<-as.matrix(imglist[[one.line[3,3]+1+backstep]])[one.line[3,1],one.line[3,2]]}
    if(depth>=4){
      white[4]<-as.matrix(imglist[[one.line[4,3]+1+backstep]])[one.line[4,1],one.line[4,2]]}
    if(depth>=5){
      white[5]<-as.matrix(imglist[[one.line[5,3]+1+backstep]])[one.line[5,1],one.line[5,2]]}
    if(depth>=6){
      white[6]<-as.matrix(imglist[[one.line[6,3]+1+backstep]])[one.line[6,1],one.line[6,2]]}
    if(depth>=7){
      white[7]<-as.matrix(imglist[[one.line[7,3]+1+backstep]])[one.line[7,1],one.line[7,2]]}
    if(depth>=8){
      white[8]<-as.matrix(imglist[[one.line[8,3]+1+backstep]])[one.line[8,1],one.line[8,2]]}
    if(depth>=9){
      white[9]<-as.matrix(imglist[[one.line[9,3]+1+backstep]])[one.line[9,1],one.line[9,2]]}
    if(depth>=10){
      white[10]<-as.matrix(imglist[[one.line[10,3]+1+backstep]])[one.line[10,1],one.line[10,2]]}
    if(depth>=11){
      white[11]<-as.matrix(imglist[[one.line[11,3]+1+backstep]])[one.line[11,1],one.line[11,2]]}
    if(depth>=12){
      white[12]<-as.matrix(imglist[[one.line[12,3]+1+backstep]])[one.line[12,1],one.line[12,2]]}
    white.mat[m,]<-white
  }
  return(white.mat)
}

muscle.plot.stl <-
function(fiber.list, res = 1, df = 2, radius = 1, cols=NULL, 
                          save.plot=FALSE, file.name="muscle.fibers", mirror.axis=FALSE,type="stl", ...){
  open3d()
  if(is.null(cols)){
    cols=rep("blue", length(fiber.list))
  }
  
  if(type=="ply"){  warning("PLY can be extremely slow, especially if saving to file!", immediate. = TRUE)
  }
  if(type=="ply" & res==1){res<-0.02}
  
  for(j in 1:length(fiber.list)){
    fiber.dat<-fiber.list[[j]]$fiber.points
    fiber.dat<-fiber.dat*res
    if(mirror.axis){
      fiber.dat[,2]<-fiber.dat[,2]*-1
    }
    fb.df <- data.frame(x = fiber.dat[, 1], y = fiber.dat[,2], z = fiber.dat[, 3])
    if(df==1){ fit <- tryCatch(lm(cbind(x,y)~z, data = fb.df))
    } else {
      fit <- tryCatch(lm(cbind(x,y)~nsp(z,df=df), data = fb.df))}
  
    if(type=="stl"){
      lines3d(cbind(predict(fit, newdata = list(z = fiber.dat[,3])), fiber.dat[,3]),col=cols[j], alpha=0.8)
    }
    if(type=="ply"){
      vertices<-cbind(predict(fit, newdata = list(z = fiber.dat[,3])), fiber.dat[,3])
      cly<- cylinder3d(vertices,color=cols[j],radius=radius*res/2,sides=8,closed = -2, ...)
      shade3d(cly) 
      wire3d(cly, col=cols[j])
      
    }
      }
    
  
  
  if(save.plot){
    if(type=="stl"){
    writeSTL(paste0(file.name,".stl"))
    }
    if(type=="ply"){
      writePLY(paste0(file.name,".ply"),withColors=TRUE)
    }
  }
  
}

boundary.check <-
function(images, new.zero.image, new.startx, new.starty, bound.buffer, cutoff){
  slice<-which(!load.image(images[new.zero.image])==0, arr.ind = T)[,1:2]
  slice[which(slice<cutoff)]<-0
  boundaries<-concaveman(slice)
  distances<-apply(boundaries,1,function(x){sqrt(sum((x-c(new.startx,new.starty))^2))})
  res<-sum(ifelse(distances<bound.buffer,1,0))
  return(res)
}

pointsGenerator <-
function(startx, starty, ucoords, radius=radius, backstep){
  
  start.points<-cbind(rep(startx,radius+1),rep(starty,radius+1),rep(0,radius+1))
  if(backstep==0){
  ucoords<-ucoords[-which(ucoords[,3]==0),]}
  line.point.list<-vector(length = nrow(ucoords),mode="list")
  
  n<-nrow(ucoords)
  
  for(i in 1:n){
    
    x<-round(seq(0,ucoords[i,1],length.out=(radius+1)),0)
    y<-round(seq(0,ucoords[i,2],length.out=(radius+1)),0)
    z<-round(seq(0,ucoords[i,3],length.out=(radius+1)),0)
    line.points<-cbind(x,y,z)
    line.point.list[[i]]<-line.points+start.points
  }
  
  return(line.point.list) 
}

fuse.fibers.auto <-
function(fiber.list,min.vox,min.improvement=0.25,df=2,length.out=50,max.iter=10, verbose=FALSE){
  new.fiber.list<-fiber.list
  iteration<-1
  repeat{
    if(iteration>max.iter){cat("max iterations reached", "\n",sep=" ")
      break()}
      if(verbose){cat("iteration #",iteration,"\n",sep=" ")}
    fiber.step<-fuse.fibers(new.fiber.list,min.vox,min.improvement,df,length.out)
    if(length(fiber.step)==1){break()}
    new.fiber.list<-fiber.step$merged.fibers
    iteration<-iteration+1
  }
  return(new.fiber.list)
  
}

fiber.lengths <-
function(fib.list, res = NULL, df = 2, length.out=500){
  nfibs<-length(fib.list)
  fib.length<-rep(NA, length=nfibs)
  for(j in 1:nfibs){
    if(nfibs==1){
      fib.points<-fib.list$fiber.points
    } else {  
      fib.points<-fib.list[[j]]$fiber.points
    }
    starts<-min(fib.points[,3])
    stops<-max(fib.points[,3])
    
    tryCatch({
      fb.df<-data.frame(x = fib.points[,1],y=fib.points[,2],z=fib.points[,3])
      newdata<-seq(starts,stops,length.out=length.out)
      if(df==1){ fit1 <- tryCatch(lm(cbind(x,y)~z, data = fb.df))
        } else {
      fit1 <- tryCatch(lm(cbind(x,y)~nsp(z,df=df), data = fb.df))}
      fib.smoothed <- cbind(predict(fit1, newdata = list(z = newdata)),newdata)
      
      fib.segs<-vector(length=(nrow(fib.smoothed)-1))
      for(i in 1:(nrow(fib.smoothed)-1)){
        fib.segs[i]<-sqrt(sum((fib.smoothed[i,]-fib.smoothed[i+1,])^2))
      }
      
      fib.length[j]<-sum(fib.segs)},error = function(msg){fib.length[j]<-NA})
    
  }
  
  
  if(!is.null(res)){
    fib.length<-fib.length*res
  }
  
  return(fib.length)
  
}

fiber.curve <-
function(fib.list, df, check=TRUE, length.out=500){
  fl<-fiber.lengths(fib.list,res=1,df=df,length.out=length.out)
  sl.dist<-rep(NA,length(fib.list))
  for(i in 1:length(fib.list)){
    fib.points<-fib.list[[i]]$fiber.points
    starts <- min(fib.points[, 3])
    stops <- max(fib.points[, 3])
    fb.df <- data.frame(x = fib.points[, 1], y = fib.points[,2], z = fib.points[, 3])
    newdata <- seq(starts, stops, length.out = length.out)
    if(df==1){ fit1 <- tryCatch(lm(cbind(x,y)~z, data = fb.df))
    } else {
      fit1 <- tryCatch(lm(cbind(x,y)~nsp(z,df=df), data = fb.df))}
    fib.smoothed <- cbind(predict(fit1, newdata = list(z = newdata)), newdata)
    endr<-nrow(fib.smoothed)
    sl.dist[i]<-sqrt(sum((fib.smoothed[1,]-fib.smoothed[endr,])^2))
    }
  
  
  curvature<-fl/sl.dist
  
  if(check){
    uplim<-boxplot.stats(curvature)$stats[5]
    iffy<-which(curvature>uplim)
    return(list(curvature=curvature,problem.fibers=iffy))
  } else {
    return(list(curvature=curvature))
  }
  
}

quality.check <-
function(fib.list,images,res=1,min.length=NULL,length.out=200, df=2){
  grey.list<-vector(length=length(fib.list),mode="list")
  
  for(i in 1:length(fib.list)){
      
    fib.points<-fib.list[[i]]$fiber.points
    starts<-min(fib.points[,3])
    stops<-max(fib.points[,3])
    fb.df<-data.frame(x = fib.points[,1],y=fib.points[,2],z=fib.points[,3])
    newdata<-seq(starts,stops,length.out=length.out)
    if(df==1){ fit1 <- tryCatch(lm(cbind(x,y)~z, data = fb.df))
    } else {
      fit1 <- tryCatch(lm(cbind(x,y)~nsp(z,df=df), data = fb.df))}
   fiber <- round(cbind(predict(fit1, newdata = list(z = newdata)),newdata),0)
    fiber<-unique(fiber)
    grey.check<-matrix(ncol=2,nrow=nrow(fiber))
    plot.index<-unique(fiber[,3])
    ncheck<-length(unique(fiber[,3]))
    
    
    
    for(k in 1:ncheck){
      tempimg<-as.matrix(load.image(images[plot.index[k]]))
      steps<-which(fiber[,3] %in% plot.index[k])
      grey.check[steps,2]<-fiber[steps[1],3]
      
      
      for(m in 1:length(steps)){
      
      x<-fiber[steps[m],1]
      y<-fiber[steps[m],2]
      if(x<=0|y<=0){next()}
      if(x>dim(tempimg)[1] | y > dim(tempimg)[2]){next()}
      grey.check[steps[m],1]<-tempimg[x,y]
      } 
    }
    grey.list[[i]]<-na.omit(grey.check)
  }
    
   
  fl<-fiber.lengths(fib.list,res=res,df=df)
  greysd<-sapply(grey.list,function(x){sd(x[,1])})/fl
  qual<-scale(greysd)
  
  p<-barplot(sort(qual,decreasing = TRUE))
  abline(h=as.numeric(quantile(qual)[4]),lty=3)
  iffy<-which(qual>(quantile(qual)[4] + IQR(qual)*1.5))
  text(p[1:length(iffy)],sort(qual[iffy],decreasing=TRUE),iffy,pos=1)
  
  if(!is.null(min.length)){
    iffy<-c(iffy,which(fl<=min.length))
  }
  
  
  return(list(quality=greysd,problem.fibers=iffy, grey.values=grey.list))
}

sequencePlot <-
function(fib.track,images,threshold=0.1,sleep.time=0.5){
  
  for(i in 1:nrow(fib.track)){
    im1<-load.image(images[fib.track[i,3]])
    im1[which(im1<threshold)]<-0
    plot(im1)
    points(fib.track[i,1],fib.track[i,2],col="red",pch=16,cex=0.75)
    Sys.sleep(sleep.time)
  }
  
}

find.endpoints<-function(images,mode="origin.insertion",show.plot=FALSE,method="kmeans"){
  slices<-vector(length=length(images),mode="list")
  for(i in 1:length(images)){
    slice <- which(!load.image(images[i]) == 0, arr.ind = T)[,1:2]
    if(dim(slice)[1]==0){slice<-matrix(NA,ncol=2,nrow=1)}
    slice<-cbind(slice,rep(i,nrow(slice)))
    slices[[i]]<-slice
  }
  slices<-do.call(rbind,slices)
  slices<-slices[-which(is.na(slices[,1])),]
  slices2<-slices
  
  if(mode=="origin.insertion"){
    if(method=="kmeans"){
    centers<-kmeans(slices,2)}
    if(method=="hclust"){
      slices<-slices[sample(1:nrow(slices),5000,replace=FALSE),]
      dist.mat<-dist(slices)
      cluster<-cutree(hclust(dist.mat,method="ward.D2"),k=2)
      centers<-list(cluster=cluster)
    }
    if(show.plot){
      plot3d(slices[,1],slices[,2],slices[,3],aspect=FALSE,col=centers$cluster)
    }
    
    end1<-apply(slices[which(centers$cluster==1),],2,mean)
    end2<-apply(slices[which(centers$cluster==2),],2,mean)
    endpoints<-rbind(end1,end2)
  } else if(mode=="tendon") {
    colnames(slices)<-c("x","y","z")
    slices<-as.data.frame(slices)
    fit<-lm(cbind(x,y) ~ z,data=slices)
    along.line<-cbind(predict(fit, newdata = list(z = slices$z)),slices$z)
    
    end1<-along.line[which(along.line[,3]==min(along.line[,3]))[1],]
    end2<-along.line[which(along.line[,3]==max(along.line[,3]))[1],]
    endpoints<-rbind(end1,end2)
  }
  colnames(endpoints)<-c("x","y","z")
  return(list(endpoints=endpoints, mask=slices2))
}

crop.stack <-
function(images, bounds=NULL, save.images=FALSE){
  `%inr%` <- imager::`%inr%`
  if(!is.null(bounds)){
    xmin<-bounds[1]
    xmax<-bounds[2]
    ymin<-bounds[3]
    ymax<-bounds[4]
  } else {
  
  boundaries<-vector(length = length(images),mode="list")
  for(i in 1:length(boundaries)){
    slice<-which(!load.image(images[i])==0, arr.ind = T)[,1:2]
    if(!is.matrix(slice)){next()}
    if(dim(slice)[1]==0){next()}
    boundaries[[i]]<-concaveman(slice)
  }
  
  xmins<-rep(NA,length(boundaries))
  xmaxs<-xmins
  ymins<-xmins
  ymaxs<-xmins
  
  for(i in 1:length(boundaries)){
    
    
    if(is.null(boundaries[[i]])){next()}
    
  xmins[i]<-min(boundaries[[i]][,1])
  xmaxs[i]<-max(boundaries[[i]][,1])
  ymins[i]<-min(boundaries[[i]][,2])
  ymaxs[i]<-max(boundaries[[i]][,2])
  
}
  xmin<-min(na.omit(xmins))-15
  xmax<-max(na.omit(xmaxs))+15
  ymin<-min(na.omit(ymins))-15
  ymax<-max(na.omit(ymaxs))+15
  
  if(xmin<0){xmin<-0}
  if(ymin<0){ymin<-0}
  }

  message("cropping images")
  for(i in 1:length(images)){
    temp<-load.image(file=images[[i]])
    
    ###### following code taken from imager::imsub
    ###### associated functions from imager
        subs<-function (im, cl, consts, envir = parent.frame()){
      if (missing(consts)) {
        consts <- list(width = width(im), height = height(im), 
                       depth = depth(im), spectrum = spectrum(im))
      }
      vl <- intersect(all.names(cl), c("x", "y", "z", "cc"))
      if (length(vl) > 1) {
        stop("Use only one of x,y,z,cc at a time")
      }
      else {
        vname <- vl
        mval <- list(x = width(im), y = height(im), z = depth(im), 
                     cc = spectrum(im))
        maxval <- mval[[vl]]
        consts[[vname]] <- 1:maxval
        inds <- eval(cl, list2env(consts, envir))
        if (vname == "x") {
          (as.array(im)[inds, , , , drop = FALSE]) %>% cimg
        }
        else if (vname == "y") {
          (as.array(im)[, inds, , , drop = FALSE]) %>% cimg
        }
        else if (vname == "z") {
          (as.array(im)[, , inds, , drop = FALSE]) %>% cimg
        }
        else {
          (as.array(im)[, , , inds, drop = FALSE]) %>% cimg
        }
      }
    }
    
    mutate_plyr<-function (.data, ...) 
    {
      stopifnot(is.data.frame(.data) || is.list(.data) || is.environment(.data))
      cols <- as.list(substitute(list(...))[-1])
      cols <- cols[names(cols) != ""]
      for (col in names(cols)) {
        .data[[col]] <- eval(cols[[col]], .data, parent.frame())
      }
      .data
    }
    
    
    
    
    im<-temp
    l <- as.list(substitute(list(x %inr% c(xmin,xmax),y %inr% c(ymin,ymax)))[-1])
    consts <- list(width = width(im), height = height(im), depth = depth(im), spectrum = spectrum(im))
    consts <- mutate_plyr(consts, cx = width/2, cy = height/2, 
                                   cz = depth/2)
    env <- new.env(parent = parent.frame())
    newim<-Reduce(function(a, b) subs(a, b, consts, envir = env), l, 
           init = im)
    ####### end code from imager::imsub
    
    if(save.images){save.image(newim,file=paste("cropped_", images[i],".png",sep=""))}
  }
  return(list(xmin =  xmin, xmax = xmax, ymin = ymin, ymax = ymax))
}

fiber.angle <-
  function(fib.list, axis=3, reference="axis",endpoints=NULL,end.to.end=FALSE){
    fangle<-rep(NA,length(fib.list))
    if(reference=="line.of.action"){axis=3}
    if(reference=="line.of.action" & is.null(endpoints)){stop("need endpoints when calculating from line of action")}
    other.axis<-setdiff(1:3,axis)
    
    if(reference=="line.of.action"){
      ######## finding rotations for fibers using endpoints
      lower<-which(endpoints[,3]==min(endpoints[,3]))
      centering<-matrix(endpoints[lower,],nrow=2,ncol=3,byrow=TRUE)
      centered<-endpoints-centering
      reference.point<-which(!apply(centered,1,sum)==0)
      v1<-sqrt(sum(centered[reference.point,c(1,3)]^2))
      
      thetay<-acos(centered[reference.point,3]/v1)
      roty<-matrix(c(cos(thetay),0,sin(thetay), 0,1,0, -sin(thetay),0,cos(thetay)),ncol=3,nrow=3,byrow=TRUE)
      rotated<-centered%*%roty
      if(rotated[reference.point,1]>0.1){
        thetay<- -thetay
        roty<-matrix(c(cos(thetay),0,sin(thetay), 0,1,0, -sin(thetay),0,cos(thetay)),ncol=3,nrow=3,byrow=TRUE)
        rotated<-centered%*%roty
      }
      
      v2<-sqrt(sum(rotated[reference.point,c(2,3)]^2))
      thetax<- acos(rotated[reference.point,3]/v2)
      rotx<-matrix(c(1,0,0, 0,cos(thetax),-sin(thetax), 0,sin(thetax),cos(thetax)),ncol=3,nrow=3,byrow=TRUE)
      rotated2<-rotated%*%rotx
      if(rotated2[reference.point,2]>0.1){
        thetax<- -thetax
        rotx<-matrix(c(1,0,0, 0,cos(thetax),-sin(thetax), 0,sin(thetax),cos(thetax)),ncol=3,nrow=3,byrow=TRUE)
        rotated2<-rotated%*%rotx
      }
      ######### rotating the fiber to the reference line
      for(i in 1:length(fib.list)){
        fib.list[[i]]$fiber.points <- (fib.list[[i]]$fiber.points-matrix(centering[1,],ncol=3,nrow=nrow(fib.list[[i]]$fiber.points),byrow=TRUE)) %*% roty %*% rotx
      }
    }
    
    
    
    for(i in 1:length(fib.list)){
      fiber.dat<-fib.list[[i]]$fiber.points
      
      if(end.to.end){
        fiber.dat<-fiber.dat[c(1,nrow(fiber.dat)),]
      }
      
      
      
      if(reference=="axis" | reference=="line.of.action"){
        fangle[i]<-angle(abs(prcomp(fiber.dat)$rotation[,1]),c(0,0,1))
      }
      
      if(reference=="plane.xz"){
        evec<-prcomp(fiber.dat)$rotation[c(1,3),1]
        if(evec[2]<0){evec<-evec*-1}
        fangle[i]<-90-angle(evec,c(1,0))
        
      }  
      if(reference=="plane.yz"){
        evec<-prcomp(fiber.dat)$rotation[c(2,3),1]
        if(evec[2]<0){evec<-evec*-1}
        fangle[i]<-90-angle(evec,c(1,0))
        
      }  
      if(reference=="plane.xy"){
        evec<-prcomp(fiber.dat)$rotation[c(1,2),1]
        if(evec[2]<0){evec<-evec*-1}
        fangle[i]<-90-angle(evec,c(1,0))
        
      }  
    }
    fangle<-unlist(fangle)
    return(fangle)
  }

hemisphere.points <-
function(radius, show.plot=FALSE, backstep=0){
  theta <- seq(0,2*pi,0.02)
  u <- seq(0,1,length.out=(radius+1)*4)
  if(!backstep==0){u<-c(-(u[((backstep)*4):2]),u)}
  
  coords<-array(dim=c(length(theta),3,length(u)))
  for(i in 1:length(u)){
    x <- sqrt(1-u[i]^2)*cos(theta)
    y <- sqrt(1-u[i]^2)*sin(theta)
    coords[,1,i]<-x
    coords[,2,i]<-y
    coords[,3,i]<-rep(u[i],length(theta))
  }
  coords<-radius*coords
  
  rcoords<-round(coords,0)
  rcoords<-cbind(as.numeric(rcoords[,1,]),as.numeric(rcoords[,2,]),as.numeric(rcoords[,3,]))
  ucoords<-unique(rcoords)
  
  if(show.plot == TRUE){
    for(i in 1:nrow(ucoords)){
      points3d(ucoords[i,1],ucoords[i,2],ucoords[i,3],col="red")
    }
  }
  return(ucoords)
}

fibers.smoothed <-
function(fib.list, df){
fiber.smoothed<-vector(mode="list",length=length(fib.list))
  
for(j in 1:length(fib.list)){
  fiber.dat<-fib.list[[j]]$fiber.points
  fb.df <- data.frame(x = fiber.dat[, 1], y = fiber.dat[,2], z = fiber.dat[, 3])
  if(df==1){ fit <- tryCatch(lm(cbind(x,y)~z, data = fb.df))
  } else {
  fit <- tryCatch(lm(cbind(x,y)~nsp(z,df=df), data = fb.df))}
  fib.smooth<-cbind(predict(fit), fiber.dat[,3])
  fiber.smoothed[[j]]<-list(fiber.points<-fib.list[[j]], fiber.smoothed<-fib.smooth)
  }
return(fiber.smoothed)
}

check.overlap <-
function(fiber.list, min.vox, df = 2){
fl<-fiber.lengths(fiber.list, df = df)
close.mat<-c(NA,NA,NA,NA)
for(i in 1:length(fiber.list)){
for(j in 1:length(fiber.list)){
if(i==j){next()}
two.fibs.list<-rbind(fiber.list[[i]]$fiber.points,fiber.list[[j]]$fiber.points)
fib1row<-nrow(fiber.list[[i]]$fiber.points)
fib2row<-nrow(fiber.list[[j]]$fiber.points)
dist.mat<-as.matrix(dist(two.fibs.list))
min.dists<-apply(dist.mat[1:fib1row,(fib1row+1):(fib1row+fib2row)],1,min)
if(mean(min.dists)<=min.vox){
  close.mat<-rbind(close.mat,c(i,j,mean(min.dists),max(dist.mat[1:fib1row,(fib1row+1):(fib1row+fib2row)])))
} else {next()}
}
}

if(!is.null(dim(close.mat))){
close.mat<-close.mat[2:nrow(close.mat),]
close.mat<-close.mat[order(close.mat[,3]),]
}


if(is.null(dim(close.mat))){message("there are no overlapping fibers\n")
  drops<-0
  return(drops)
} else {
  drops<-c(NA,NA)
    for(i in 1:nrow(close.mat)){
      if(i>1){
        if(close.mat[i,1] %in% drops[,2] | close.mat[i,2] %in% drops[,2]){next()}}
      check<-fl[close.mat[i,1]]-fl[close.mat[i,2]]
      drops<-rbind(drops, 
                   c(ifelse(check>=0,close.mat[i,1],close.mat[i,2]),
                     ifelse(check>=0,close.mat[i,2],close.mat[i,1])))
    }
  
colnames(drops)<-c("keep","discard")


drops<-drops[2:nrow(drops),]
return(list(drop.fibers=sort(unique(drops[,2])),overlapping.fibers=drops, fibers.removed=fiber.list[-unique(drops[,2])]))
}
}

\name{muscle.plot.multi}
\alias{muscle.plot.multi}
\title{
Plot multi muscle fibers with a muscle outline
}
\description{
Uses functions from rgl to plot all fibers (smoothed with splines) in a fiber list. Also uses grayscale values from the image stack to determine the external boundaries of the muscle based on concave hulls. Boundaries are plotted as a series of single outlines sampled across the image.

}
\usage{
muscle.plot.multi(fiber.list, images, df = 2, outline = 30, 
cols = NULL, mirror.axis = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{fiber.list}{
A list of fibers with $fiber points. Generated by good.fibes or processed cleaned by other functions}
  \item{images}{
A character vector of image stack file names. Generated with list.files}
  \item{df}{
The degrees of freedom to pass to splines2::nsp for smoothing fiber paths. df = 1 gives a straight path, while >1 gives increasingly curved paths}
  \item{outline}{
The number of wireframe "outlines" to draw the muscle boundaries
}
  \item{cols}{
An optional vector of colors, the same order and length of fiber.list}
  \item{mirror.axis}{
Depending on the way the image stack was exported, fibers may be reflected from their original original. mirror.axis = TRUE will reflect the fibers before plotting to correct this
}
}
\value{
Returns a 3D plot
}

\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030.}
\author{
J. Arbour
}
\seealso{
\code{\link{muscle.plot}}, \code{\link{muscle.plot.stl}}
}
\examples{

\donttest{

olddir<-getwd()

#### this downloads the ant dataset image stack
#### if you have it already downloaded you can navigate to that folder
setwd(tempdir())
download.file(url=
"https://github.com/jessica-arbour/Ant-Muscle-Image-Stack/raw/main/Ant_data.zip",
destfile="antdata.zip")

unzip("antdata.zip")
setwd(paste0(getwd(),"/Ant data"))
####

images<-list.files(pattern=".png")
data(ant.final)
muscle.plot.multi(ant.final, images, df=1, mirror.axis = TRUE)


setwd(olddir)
}

}

\name{fibers.smoothed}
\alias{fibers.smoothed}
\title{
Smoothing of estimated fiber paths}
\description{
Applies splines to smooth the stepwise fiber paths produced by good.fibes, with the function \emph{ns} from \strong{splines}.
}
\usage{
fibers.smoothed(fib.list, df)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{fib.list}{
A list of fibers with $fiber.points, produced by \code{\link{good.fibes}} or one of the cleaning and processing functions (e.g., fuse.fibers).
}
  \item{df}{
The degrees of freedom passed to splines2::nsp. Knots equal to df - 1 - intercept are set as breakpoints in the spline curve. A straight line path has a df of 1.
}
}
\value{
\item{fiber.points}{The original fiber path from good.fibes}
\item{fiber.smoothed}{The curved, smoothed fiber paths}
}
\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030..
}
\author{
J. Arbour}
\seealso{
\code{\link{good.fibes}}
}

\name{fuse.fibers.auto}
\alias{fuse.fibers.auto}
\title{
Automated repeated fusing of fiber paths}
\description{
This function repeatedly compares pairs of muscle fibers for fusing. Repeated applications of fuse.fibers until no further fibers can be merged}
\usage{
fuse.fibers.auto(fiber.list, min.vox, min.improvement = 0.25, 
df = 2, length.out = 50, max.iter = 10, verbose = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{fiber.list}{
A list of fibers with $fiber.points produced by good.fibes or any of the processing and cleaning functions.
}
  \item{min.vox}{
The voxel distance below which fibers will be compared. Should be <= the voxel width of the muscle fascicles, though lower if interstital spaces are low.
}
  \item{min.improvement}{
The minimum increase (as a proportion) in fiber length for fibers to be worth merging.
}
  \item{df}{
The df to be used in smoothing fiber paths in the calculation of fiber length
}
  \item{length.out}{
The number of divisions to be used in the calculation of fiber lengths (line segments)
}
  \item{max.iter}{
The maximum number of iterations of fuse.fibers that will be attempted.
}
  \item{verbose}{
Should the number of iterations through the function be displayed while running?
}
}
\value{
A list of fibers with $fiber.points
}
\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030.
}
\author{
J. Arbour
}
\seealso{
\code{\link{fuse.fibers}}
}

\name{get.whites}
\alias{get.whites}
\title{
An internal function determining grayscale values}
\description{
Used internally in good.fibes to determine grayscale values along all possible line paths
}
\usage{
get.whites(imglist, lpl, depth, backstep = 0)
}
\arguments{
  \item{imglist}{
The current images being considered
}
  \item{lpl}{
generated by points.generator to find the position of all possible paths
}
  \item{depth}{
radius + 1 from good.fibes
}
  \item{backstep}{
How many images to consider "behind" the plane of the zero.image. 
}
}
\details{
\strong{for internal use in good.fibes only}
}
\value{
A matrix of grayscale values > cutoff
}
\keyword{internal}

\name{thresholdPlot}
\alias{thresholdPlot}
\title{
Plot image from diceCT stack using thresholding
}
\description{
Plot a selected image from the image stack with values below \emph{threshold} set to black (grayscale = 0). Can be used to select \emph{threshold} and \emph{cutoff} values used in good.fibes.
}
\usage{
thresholdPlot(images, n, threshold)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{images}{
A character vector of image stack file names. Generated with list.files
}
  \item{n}{
The number of the selected image in the vector "images"
}
  \item{threshold}{
The cutoff value for grayscale values. All voxels with grayscales below threshold will be displayed as black.
}
}
\value{
Returns a plot
}

\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030.}
\author{
J. Arbour
}
\seealso{
\code{\link{sequencePlot}}
}
\examples{
\donttest{
images <- dir(system.file("extdata", package = "GoodFibes"), ".png", full.names = TRUE)

thresholdPlot(images, 1, 0.3)
thresholdPlot(images, 1, 0.4)
thresholdPlot(images, 1, 0.5)
}
}

\name{pointsGenerator}
\alias{pointsGenerator}
\title{
Line points generator
}
\description{
An internal function for generating line coordinates for all possible paths of the good.fibes tracking algorithm.
}
\usage{
pointsGenerator(startx, starty, ucoords, radius = radius, backstep)
}
\arguments{
  \item{startx}{
seed point x coordinate
}
  \item{starty}{
seed point y coordinate
}
  \item{ucoords}{
unique end coordinates generated by hemisphere.points
}
  \item{radius}{
Number of images to consider forward from the zero.image
}
  \item{backstep}{
Should images in the seed point plane or behind it be considered. Each value (1, 2, 3, etc.) gives the number of images "behind" the seed point to consider in the spherical dome
}
}
\details{
For internal use in good.fibes only}
\value{
A list containing coorindates for each line ending in the end points determined by hemisphere.points
}
\keyword{ ~internal }

\name{fiber.curve}
\alias{fiber.curve}
\title{
Calculate the relative curvature of muscle fibers
}
\description{
Calculates a metric for fiber curvature. This is the ratio between the total length of the curved smoothed fiber, to the straight line distance between the end points of the fiber. A straight fiber will have a curvature value of ~ 1 (small differences may be due to the calculation of fiber length across a smoothed curve), and values > 1 represent more curvature.

Optionally identified which fibers show unusual curvature (are outliers), for possible removal.
}
\usage{
fiber.curve(fib.list, df, check = TRUE, length.out=500)
}
\arguments{
  \item{fib.list}{
A list of fibers containing $fiber.points. Produced by good.fibes or the various cleaning functions (quality check, fuse.fibers, check.overlap)}
  \item{df}{
Corresponds to the df argument in splines2::nsp. Determines the shape of the smoothing spline (df = 1 represents straight muscle fibers)}
  \item{check}{
Should unusually curved fibers be identified?}
  \item{length.out}{
The number of straight line segments that the smoothed curve will be divided into for calculation of length
}
}
\value{

\item{curvature}{The ratio of fiber length to end-to-end length}
\item{problem.fibers}{Fibers with unusually high curvature. Given as the index of these fibers in the original list.}
}
\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030..
}
\author{
J. Arbour
}
\seealso{
\code{\link{fiber.lengths}},\code{\link{fiber.angle}}
}
\examples{

data(ant.final)

\donttest{fcr<-fiber.curve(ant.final,df=2,check=TRUE) 
#fibers reconstructed with a curve here merely to demonstrate function
#ant fibers were fairly straight

sort(fcr$curvature) 
#all fibers are close to 1 even with a "curved" reconstruction}
}

\name{check.overlap}
\alias{check.overlap}
\title{
Check if muscle fibers are redundant}
\description{
Determines whether muscle fibers are likely to be repeats. The function compares pairs of fibers and determines 1) whether they are within min.vox of each other, and 2) if the average distance between them stays less than min.vox. If so the longer fiber is kept and the redundant fiber is dropped.}
\usage{
check.overlap(fiber.list, min.vox, df = 2)
}
\arguments{
  \item{fiber.list}{
A list containing elements with $fiber.points. Generated by \code{\link{good.fibes}}, \code{\link{fibers.smoothed}}, \code{\link{quality.check}}, or \code{\link{fuse.fibers}}.
}
  \item{min.vox}{
The distance between fibers in voxels (pixels) for fibers to be considered redundant}
  \item{df}{
The degree of curvature for spline interpolation via splines::ns
}
}
\value{

\item{drop.fibers}{the index for the redundant fibers to be removed}

\item{overlapping.fibers}{a matrix with the compared fibers, the index of which were kept and which were removed in each comparison}

\item{fibers.removed}{a list with fiber.points with the redundant fibers excluded}
}
\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030.}
\author{
J. Arbour}
\seealso{
\code{\link{fuse.fibers}},\code{\link{quality.check}},\code{\link{good.fibes}}
}

\name{fiber.angle}
\alias{fiber.angle}
\title{
Calculating the orientation of muscle fibers}
\description{
Calculating the orientation or pennation angle of individual muscle fibers. This function can calculate fiber angles around a central axis (x, y or z) or in reference to a particular "view" (plane). Fiber data can optionally be aligned to a line of action (tendon, line between origin and insertion points, etc.) to calculate pennation angle.
}
\usage{
fiber.angle(fib.list, axis, reference = "axis", 
endpoints=NULL, end.to.end=FALSE)
}
\arguments{
  \item{fib.list}{
A list of muscle fiber paths generated by good.fibes or from the various cleaning and processing function (must contain $fiber.points).
}
  \item{axis}{
The axis around which angles will be calculated as a deviation from. Default is 3 (z). This axis is treated as the "z" axis for "plane" options.
}
   \item{reference}{
Should the orientation angles be calculated about an axis (specified by "axis") or in reference to a plane ("plane.xz" "plane.yz", or "plane.xy", where z is the value of "axis"). When calculating pennation angles, reference = "line.of.action" and argument "endpoints" should be supplied.
}
\item{endpoints}{
An optional 2X3 matrix generated by \code{\link{find.endpoints}}. When supplied, data is centered on the first endpoint and rotated such that the second lies along the z axis, such that angles are calculated in reference to a straight line between the two endpoints.
}
\item{end.to.end}{
Should the angle be calculated over the entire fiber (FALSE) or based on just the enpoints of the fiber (TRUE).
}
}
\details{
Fiber angle is calculated from the eigenvectors of a Principal Component Analysis of the fiber coordindates using \code{\link{prcomp}}.
}
\value{
A vector of angles in degrees corresponding to each fiber in the original list}
\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030.

Katzke, J., Puchenkov, P., Stark, H., and Economo, E. 2022. A Roadmap to Reconstructing Muscle Architecture from CT Data. \emph{Integrative Organismal Biology} 4(1): 1-16.

Sullivan, S., McGechie, F., Middleton, K., and Holliday, C. 3D Muscle Architecture of the Pectoral Muscles of European Starling (Sturnus vulgaris).\emph{Integrative Organismal Biology} 1(1):1-18.

}
\author{
J. Arbour
}
\seealso{
\code{\link{fiber.lengths}}
}
\examples{

\donttest{
olddir<-getwd()

data(ant.final)
fangle<-fiber.angle(ant.final,3)
fangle

cols<-color.scale(fangle, c("blue", "red"))
muscle.plot.stl(ant.final, cols=cols, df = 1)

setwd(olddir)
}
}

\name{ant.final}
\alias{ant.final}
\docType{data}
\title{
Ant muscle fibers finalized
}
\description{
Muscle fibers reconstructed from the mandibular muscle of an ant \emph{(Monomorium pharaonis)} (post processing).
}
\format{
A list with 198 entries, each representing a muscle fiber reconstructed from an image stack. Each entry contains $fiber.points, the raw reconstructed fiber paths, and $fiber.smoothed, the smoothed paths. 
}
\details{
These fibers were reconstructed using \emph{good.fibes}, checked for poor fiber paths using \emph{quality.check}, and had fibers fused and merged using \emph{fuse.fibers} and \emph{check.overlap}. Also see "ant.raw" for the initial fiber paths from good.fibes.
}
\source{
Fiber paths were generated from the ant dataset in Katzke et al (2022). Original image files available upon request.
}
\references{
Katzke, J., Puchenkov, P., Stark, H., and Economo, E. 2022. A Roadmap to Reconstructing Muscle Architecture from CT Data. \emph{Integrative Organismal Biology} 4(1): 1-16.
}
\examples{
\donttest{
data(ant.final)
fl<-fiber.lengths(ant.final, res = 0.000673107, df = 1)
}

}
\keyword{datasets}

\name{muscle.plot}
\alias{muscle.plot}
\title{
Plot a single muscle fiber}
\description{
Used to compare the muscle fiber path to the smoothed muscle fiber. Plots a single set of $fiber.points from and the smoothed fibers.
}
\usage{
muscle.plot(fiber.dat, images, df = 4, mirror.axis = FALSE, outline = 50, size = 2)
}
\arguments{
  \item{fiber.dat}{
Any set of $fiber.points produced by good.fibes
}
  \item{images}{
A character vector with image names representing the image stack, can be produced using list.files.
}
  \item{df}{
The df to be used in smoothing fiber paths in the calculation of fiber length
}
  \item{mirror.axis}{
Depending on the way the image stack was exported, fibers may be reflected from their original original. mirror.axis = TRUE will reflect the fibers before plotting to correct this
}
  \item{outline}{
The number of wireframe "outlines" to draw the muscle boundaries
}
  \item{size}{
point size for $fiber.points in plot
}
}
\value{
Returns a 3D plot
}

\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030.}
\author{
J. Arbour
}
\seealso{
\code{\link{muscle.plot.multi}}, \code{\link{muscle.plot.stl}}
}
\examples{
\donttest{
olddir<-getwd()

#### this downloads the ant dataset image stack
#### if you have it already downloaded you can navigate to that folder
setwd(tempdir())
download.file(url=
"https://github.com/jessica-arbour/Ant-Muscle-Image-Stack/raw/main/Ant_data.zip",
destfile="antdata.zip")

unzip("antdata.zip")
setwd(paste0(getwd(),"/Ant data"))
####

images<-list.files(pattern=".png")

data(ant.final)
muscle.plot(ant.final[[100]]$fiber.points,images,df=1, outline=30, mirror.axis=TRUE)
setwd(olddir)
}
}

\name{ant.raw}
\alias{ant.raw}
\docType{data}
\title{
Raw ant muscle fibers
}
\description{
Preliminary muscle fibers detected from the mandibular muscle of an ant \emph{(Monomorium pharaonis)}. No quality checking or processing.
}
\usage{data("ant.raw")}
\format{
A list with 247 entries, each representing a muscle fiber reconstructed from an image stack (dataset available in examples below) and a partial, subsampled stack is available in extdata. Each entry contains $fiber.points, the raw reconstructed fiber paths, and $fiber.smoothed, the smoothed paths. }
\details{
These are the initial possible fibers detected using \code{\link{good.fibes}}. Also see \emph{ant.final} for the processed fibers.
}
\source{
Fiber paths were generated from the ant dataset in Katzke et al (2022). Original image files are stored under extdata (and see example).
}
\references{
Katzke, J., Puchenkov, P., Stark, H., and Economo, E. 2022. A Roadmap to Reconstructing Muscle Architecture from CT Data. \emph{Integrative Organismal Biology} 4(1): 1-16.
}
\examples{

\donttest{
data(ant.raw)
fl<-fiber.lengths(ant.raw, res = 0.000673107, df = 1)

#### this downloads the ant dataset image stack to a temp folder
olddir<-getwd()
setwd(tempdir())
download.file(url=
"https://github.com/jessica-arbour/Ant-Muscle-Image-Stack/raw/main/Ant_data.zip",
destfile="antdata.zip")

unzip("antdata.zip")
setwd(paste0(getwd(),"/Ant data"))

setwd(olddir)
}
}
\keyword{datasets}

\name{hemisphere.points}
\alias{hemisphere.points}
\title{
Finding end points of possible paths
}
\description{
Internal function that projects a hemisphere or spherical dome through the image stack to determine the end voxels of each possible path from a seed point.
}
\usage{
hemisphere.points(radius, show.plot = FALSE, backstep = 0)
}
\arguments{
  \item{radius}{
The number of images out from the seed point plane to sample
}
  \item{show.plot}{
Generates a 3D plot showing the end points of possible paths
}
  \item{backstep}{
Should images in the seed point plane or behind it be considered. Each value (1, 2, 3, etc.) gives the number of images "behind" the seed point to consider in the spherical dome
}
}
\value{
Returns a matrix with 3D coordinates of the end points of the possible paths
}
\author{
J. Arbour
}

\seealso{
\code{\link{pointsGenerator}}
}

\keyword{internal}

\name{good.fibes}
\alias{good.fibes}
\title{
Automated detection of muscle fibers from diceCT scans
}
\description{
The function good.fibes uses textural analysis to determine the path of muscle fibers/fascicles through an image stack from a iodine contrast CT scan. Fiber paths are reconstructed using a stepwise algorithm that follows paths of low variation in threshold values. See details for full description of the method

\strong{SOMETIMES ABORTS RSTUDIO BUT STILL RUNS IN R-GUI, PROBLEM IS ONLY IN RSTUDIO}
}
\usage{
good.fibes(images, zero.image, radius, threshold = NULL, 
cutoff, scaler = 1, blackcut = 0.95, seeds = 1, show.plot = TRUE, 
start.seed = NULL, allowed.black = 0, bound.buffer = 0, backstep = 0, verbose=TRUE)
}
\arguments{
  \item{images}{
A character vector with image names representing the image stack from a diceCT scan. The voxels are assumed to be isometric, and the images should be in .png format. The vector can be produced using list.files(pattern = ".png")
}
  \item{zero.image}{
The number of the image in the stack from which seed points should be drawn. Only one image can be selected.
}
  \item{radius}{
The number of images to consider forward or backward from the zero.image at each step of the walk. Maximum 11.
}
  \item{threshold}{
The grayscale value below which voxels will be considered black for the selection of seed points. Must be equal to or greater than cutoff}
  \item{cutoff}{
The grayscale value below which voxels will be considered black in the forwards and backwards walk. Use \code{\link{thresholdPlot}} to determine a value that isolates muscle voxels from other tissues/background noise.
}
  \item{scaler}{
Exponential scaler for the trajectory penalization. Default is 1. At a value of 0 there is no trajectory penalization
}
  \item{blackcut}{
A termination condition. If a specified percentage (as proportion, e.g., 0.95) of voxels in the hemisphere of paths are black, the algorithm will terminate. 
}
  \item{seeds}{
The number of seed points on the starting image. The seed points will generate a possible fiber path, if a walk is possible (seeds can fail if they are located on noise without possible paths).
}
  \item{show.plot}{
Optionally show the location of the tracker in the image stack at each step
}
  \item{start.seed}{
Optionally applies \emph{set.seed} in the tracker to make results reproducible from one run to the next. See set.seed for more details.
}
  \item{allowed.black}{
For noisy datasets, allows this number of voxels with grayscale values below \emph{cutoff} to be included in the possible paths without terminating the algorithm
}
  \item{bound.buffer}{
If a fiber path is within this many voxels distance of the outermost boundary of the muscle, as determined by grayscale values above the \emph{cutoff}, then the algorithm will terminate the particular path. See details.
}
  \item{backstep}{
How many images "behind" the current plane should be considered. Should be kept to low values (1-3). When backstep = 0, only paths ending on images "ahead" of the image plane will be considered. \strong{EXPERIMENTAL}, will create some weird paths. Use only if the muscle fibers definitely arc back through the image stack and perhaps only on image planes close to that point.
}
  \item{verbose}{
If TRUE will list the progress through each fiber}

}
\details{
The function begins by selecting a set of seed points from the selected image. Grayscale values below \emph{threshold} are excluded, and the pairwise euclidean distances among all remaining voxels are calculated. Cluster analysis is conducted using \emph{hclust} and a set of groups equal to \emph{seeds} are produced using \emph{cutree}. Voxels within each group are randomly selected.

From a selected seed voxel, a hemisphere of possible paths is projected, extending \emph{radius} images from the selected starting image. If \emph{backstep} is >0, paths within the starting image plane and behind the plane (1 = 1 image behind, 2 = 2 images behind) are also included in the possible paths. \strong{NOTE} backstep is experimental and does cause more circuitous paths, use only if fiber paths reverse direction through the image stack at some point.

The forward walk from the seed point begins by choosing from the set of possible paths, the one that minimizes the following function.

diagnostic value = scaled grayscale SD * trajectory ^ scaler 

1)The scaled grayscale SD is the standard deviation of grayscale values along each possible path. This value is scaled to 0 to 1. 
2)The trajectory is the straight line distance between the end points on the hemisphere between the previous step and the next possible steps. This value is scaled to a range of 0 to 1, and added to 1 (resulting values range from 1 to 2). This is to penalize steps that make severe changes, as muscle fibers tend to not have very severe bends. 
3)The impact of the trajectory penalization is scaled using \emph{scaler}. If scaler = 0, there is no trajectory penalization.

The path with the minimum diagnostic value is selected, and the process repeated from the end point of that path. This stepwise algorithm continues the forward walk through the image stack until one of several stop conditions is reached:

1) The only available remaining paths would either terminate or cross a black voxel. This prevents the tracker from passing out of the muscle fascicle. To accommodate noisy datasets, the tracker may be permitted to cross a small number of black voxels (\emph{allowed.black}). 
2) The number of black voxels in the possible paths exceeds a specified number (e.g., 95 percent). This is meant to isolate regions of noise towards the end of a muscle fascicle, where adjoining connective tissue may obscure the end of a fiber. 
3) The remaining paths would terminate within a specified distance of the external "boundary" of the muscle. This prevents fibers from continuing to track along the exterior surface of the muscle in noisy image stacks.

Once the path is terminated, the algorithm returns to the seed point and begins a walk in the opposite direction. It proceeds using the same terms as above. The forwards and backwards walks are returned. The process repeats for the next seed point.

}
\value{
A list with a length equal to or less than seeds (failed paths will be dropped). Each element contains $fiber.points, the 3D coordinates providing the fiber path through the image stack, expressed in units of voxels.

Can be combined with separate runs from other images planes using c(). See example
}
\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030.
}
\author{
J. Arbour
}
\seealso{
\code{\link{fibers.smoothed}}
}
\examples{

\donttest{

olddir<-getwd()

#### this downloads the ant dataset image stack
#### if you have it already downloaded you can navigate to that folder
setwd(tempdir())
download.file(url=
"https://github.com/jessica-arbour/Ant-Muscle-Image-Stack/raw/main/Ant_data.zip",
destfile="antdata.zip")

unzip("antdata.zip")
setwd(paste0(getwd(),"/Ant data"))
####


images<-list.files(pattern=".png")

fibes1<-good.fibes(images = images, zero.image = 200, radius = 9, threshold = 0.7,
cutoff = 0.65, seeds=5, start.seed = 1, show.plot=FALSE)

fibes2<-good.fibes(images = images, zero.image = 300, radius = 9, threshold = 0.7,
cutoff = 0.65, seeds=5, start.seed = 1, show.plot=FALSE)

fibes<-c(fibes1,fibes2)

muscle.plot.multi(fibes, images, df=1)


setwd(olddir)
}

}

\name{muscle.plot.stl}
\alias{muscle.plot.stl}
\title{
Plot and export muscle fibers to stl}
\description{
Plot a series of muscle fibers produced by good.fibes. Fibers are smoothed using splines before plotting. Optionally export an STL file in the correct size scale.}
\usage{
muscle.plot.stl(fiber.list, res = 1, df = 2, radius = 1, cols = NULL,
save.plot = FALSE, file.name = "muscle.fibers", 
mirror.axis = FALSE, type = "stl", ...)
}
\arguments{
  \item{fiber.list}{
A list of fibers with $fiber points. Generated by good.fibes or processed cleaned by other functions
}
  \item{res}{
The isometric resolution of the original scan (i.e., the distance between images). Provided as a linear measure (um, mm, etc.). Arbitrarily set to 0.02 if "ply" option is selected AND default of res=1 is used, to help with calibrating cylinder3d size.
}
  \item{df}{
The degrees of freedom to pass to splines2::nsp for smoothing fiber paths. df = 1 gives a straight path, while >1 gives increasingly curved paths
}
  \item{radius}{
The radius of the lines plotted for muscle fibers
}
  \item{cols}{
An optional vector of colors, the same order and length of fiber.list
}
  \item{save.plot}{
When TRUE, plot is saved as an .stl object in the current working directory. Provide file in file.name argument.
}
  \item{file.name}{
Character data giving the file.name, not inclulding the file extension (which will be determined by type).
}
  \item{mirror.axis}{
Depending on the way the image stack was exported, fibers may be reflected from their original orientation. mirror.axis = TRUE will reflect the fibers before plotting to correct this
}
 \item{type}{
Options are "stl" and "ply". Option "ply" preserves color selections per fiber and saves as a .ply file, but may be large and load slowly. Option "stl" uses a simplier plotting option but will not preserve color information in the .stl file.
}
\item{...}{
Arguments to be passed to "cylinder3d" in "rgl".
}

}
\value{
Returns a 3D plot
}

\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030.}
\author{
J. Arbour
}
\seealso{
\code{\link{muscle.plot.multi}},\code{\link{muscle.plot}},\code{\link{good.fibes}}
}
\examples{

data(ant.final)
\donttest{
muscle.plot.stl(ant.final, res = 0.000673107, df=1, radius = 1, 
mirror.axis = TRUE, save.plot = FALSE)
}
}

\name{equalize.stack}
\alias{equalize.stack}
\title{
Automated histogram equalization of image state
}
\description{
Conducts histogram equalization to adjust the contrast of the image stack. May improve visibility of muscle fibers prior to fiber detection. Optionally automatically save new image stack in working directory.
}
\usage{
equalize.stack(images, n, save.images = FALSE)
}
\arguments{
  \item{images}{
A vector of png image file names, created using list.files
}
  \item{n}{
The number of the image in the stack to be equalized and plotted
}
  \item{save.images}{
Should the whole image stack be equalized and plotted?
}
}
\value{
Creates a plot and optionally saves an image stack
}
\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030.}
\author{
J. Arbour
}
\seealso{
\code{\link{crop.stack}}
}
\examples{
\donttest{
olddir<-getwd()

#### this downloads the ant dataset image stack
#### if you have it already downloaded you can navigate to that folder
setwd(tempdir())
download.file(url=
"https://github.com/jessica-arbour/Ant-Muscle-Image-Stack/raw/main/Ant_data.zip",
destfile="antdata.zip")

unzip("antdata.zip")
setwd(paste0(getwd(),"/Ant data"))
####

images<-list.files(pattern=".png")
equalize.stack(images, 100)  


setwd(olddir)
}
}

\name{fiber.lengths}
\alias{fiber.lengths}
\title{
Calculated the length of smoothed muscle fibers}
\description{
Determines the length of reconstructed and smoothed muscle fibers. Fibers are smoothed using splines::ns and then oversampled (length.out). The sum of all straight line segments on the smoothed paths is taken as the overall fiber length}
\usage{
fiber.lengths(fib.list, res = NULL, df = 2, length.out = 500)
}
\arguments{
  \item{fib.list}{
A list of fibers containing $fiber.points. Produced by good.fibes or the various cleaning functions (quality check, fuse.fibers, check.overlap)}
  \item{res}{
The resolution of the isometric voxels (i.e., the distance between images). Should be given as a linear measure (e.g., mm, um)}
  \item{df}{
The degrees of freedom passed to splines2::nsp. A df = 1 produces a straight fiber, while values >1 allow fibers to curve.}
  \item{length.out}{
The number of straight line segments that the smoothed curve will be divided into for calculation of length
}
}
\value{
A vector with fiber lengths
}
\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030..
}
\author{
J. Arbour}

\seealso{
\code{\link{fiber.angle}},\code{\link{fiber.curve}}
}
\examples{

data(ant.final)

\donttest{
fl<-fiber.lengths(ant.final, res = 0.000673107, df=1)
mean(fl)
}
}

\name{sequencePlot}
\alias{sequencePlot}
\title{
Plot the path of a muscle fiber generated using good.fibes}
\description{
Plots images in sequence showing the image stack and the location of the muscle fiber path at each step in the fiber tracking algorithm in good.fibes
}
\usage{
sequencePlot(fib.track, images, threshold = 0.1, sleep.time = 0.5)
}
\arguments{
  \item{fib.track}{
A set of $fiber.points from a fiber list, generated by good.fibes
}
  \item{images}{
A character vector of image stack file names. Generated with list.files
}
  \item{threshold}{
A cutoff values under which voxels are set to black
}
  \item{sleep.time}{
Time in seconds between images, sets speed for plotting sequence
}
}
\value{
Returns a sequence of plots
}

\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030.}
\author{
J. Arbour
}
\seealso{
\code{\link{thresholdPlot}}
}
\examples{
\donttest{

olddir<-getwd()

#### this downloads the ant dataset image stack
#### if you have it already downloaded you can navigate to that folder
setwd(tempdir())
download.file(url=
"https://github.com/jessica-arbour/Ant-Muscle-Image-Stack/raw/main/Ant_data.zip",
destfile="antdata.zip")

unzip("antdata.zip")
setwd(paste0(getwd(),"/Ant data"))
####

images<-list.files(pattern=".png")

data(ant.raw)

sequencePlot(ant.raw[[2]]$fiber.points, images, 0 ,0.2)



setwd(olddir)
}

}

\name{boundary.check}
\alias{boundary.check}
\title{
Check if muscle fiber is close to boundary of muscle}
\description{
An internal function that uses a concavehull to determine the bounds of the muscle based on grayscale values on each image and determines how many boundary voxels are within a set distance from the selected image point.
}
\usage{
boundary.check(images, new.zero.image, new.startx, new.starty, bound.buffer, cutoff)
}
\arguments{
  \item{images}{
A vector of image names in the working directory, created using list.files
}
  \item{new.zero.image}{
The image plane being considered
}
  \item{new.startx}{
The x coordinate of the considered point
}
  \item{new.starty}{
The y coordinate of the considered point
}
  \item{bound.buffer}{
The number of voxels a point can be distant from the boundary
}
  \item{cutoff}{
The grayscale value under which voxels will be treated as black
}
}
\details{
For internal use only
}
\value{
The number of boundary voxels within bound.buffer distance of the proposed point.
}
\author{
J. Arbour
}
\keyword{internal}

\name{fuse.fibers}
\alias{fuse.fibers}
\title{
Fuse incomplete fibers}
\description{
This function compares fibers that pass between a minimum number of voxels and determines if merging them into a single fiber produces a well supported path}
\usage{
fuse.fibers(fiber.list, min.vox, min.improvement = 0.25, df = 2, length.out = 100)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{fiber.list}{
A list of fibers with $fiber.points produced by good.fibes or any of the processing and cleaning functions.
}
  \item{min.vox}{
  The voxel distance below which fibers will be compared. Should be <= the voxel width of the muscle fascicles, though lower if interstital spaces are low.
}
  \item{min.improvement}{
The minimum increase (as a proportion) in fiber length for fibers to be worth merging.
}
  \item{df}{
The df to be used in smoothing fiber paths in the calculation of fiber length
}
  \item{length.out}{
The number of divisions to be used in the calculation of fiber lengths (line segments)
}
}
\details{
This function compares pairs of fibers if they come within mix.vox of each other along their path. The fibers will be merged if 1) the mean 3D residual from the new spline through the combined fiber path is less than the mean residual from the two separate fiber paths, and 2) the fiber length of the combined fiber is at least min.improvement (proportionately) greater than the previous fiber lengths.
}
\value{
\item{merged.fibers}{A list of fibers with $fiber.points, with fibers combined based on above thresholds}
\item{fibers.to.merge}{a matrix of pairs of fibers that were merged}
}
\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030.}
\author{
J. Arbour
}
\seealso{
\code{\link{quality.check}}, \code{\link{check.overlap}}
}

\name{find.endpoints}
\alias{find.endpoints}
\title{
Find endpoints for the line of action of a muscle
}
\description{
Uses a binary mask of either a tendon or the origin and insertion sites of a muscle. Used by \code{\link{fiber.angle}} to calculate the pennation angle of fibers. 
}
\usage{
find.endpoints(images, mode = "origin.insertion", show.plot = FALSE, method = "kmeans")}
\arguments{
  \item{images}{
A vector of png image file names, created using list.files. Should include a binary mask.
}
  \item{mode}{
The coordinates of all white values from the image mask are obtained. If "origin.insertion", first uses k-means or hclust to automatically identify two clusters representing the origin and insertion sites. The mean of each of these clusters is taken and returned in endpoints. 
If "tendon"  is selected, a line is fit through the coordinates of the mask and the fitted values from the highest and lowest z-value of the mask are returned.
}
  \item{show.plot}{
Optionally show the two clusters generated by k-means for the "origin.insertion" option. Check that these are sensible.
}
  \item{method}{
Either "kmeans" or "hclust". "hclust" option first downsamples the mask to 5000 points, then conducts hierarchical cluster analysis using a Eucliedean distance matrix and Ward's algorithm. Default "kmeans" is often faster but may not always be effective if regions are variable in size or very wide.
}

}
\value{
 \item{endpoints}{
A 2X3 matrix of coordinates featuring the endpoints of a line to rotate the muscle fibers. To be used in \code{\link{fiber.angle}} to determine pennation angles around a line of action.
}
 \item{mask}{
The coordinates representing the binary mask in "images"
}
}
\details{
"images" must have the exact same dimensions (pixel width and height, image slice position) and orientation of the images used for fiber tracking. If cropped, the exact same settings should have been used on both image stacks.
}
\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030.}
\author{
J. Arbour
}
\seealso{
\code{\link{fiber.angle}}
}

\name{GoodFibes-package}
\alias{GoodFibes-package}
\alias{GoodFibes}
\docType{package}
\title{
\packageTitle{GoodFibes}
}
\description{
\packageDescription{GoodFibes}
}
\details{

The DESCRIPTION file:
\packageDESCRIPTION{GoodFibes}
\packageIndices{GoodFibes}
Functions in this package allow for the reconstruction of muscle fibers from diceCT image stacks. Fibers are detected using textural analysis, smoothed using splines and processed for path quality (grayscale variation). Muscle fibers can be plotted in 3D with or without the overall muscle outline, and the 3D muscle fibers can be exported as an STL file. Basic fiber architecture metrics can be calculated.}
\author{
\packageAuthor{GoodFibes}

Maintainer: \packageMaintainer{GoodFibes}
}
\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030.
}
\keyword{ package }

\name{quality.check}
\alias{quality.check}
\title{
Quality testing of possible muscle fibers detected by good.fibes}
\description{
Calculates quality as the ratio of grayscale standard deviation and fiber length for each muscle fiber detected using good.fibes. Long, homogenous fibers are considered to be of higher quality. Fibers are smoothed before the calculation of fiber quality.

Fibers with usually low quality (high grayscale variation compared to fiber length) are identified for exclusion.

}
\usage{
quality.check(fib.list, images, res, min.length = NULL, length.out = 200, df = 2)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{fib.list}{
A list of muscle fibers with $fiber.points, generated by good.fibes
}
  \item{images}{
A character vector of image stack file names. Generated with list.files
}
  \item{res}{
The isometric resolution of the voxels (i.e., the distance between images). Given as a linear measure (um, mm, etc.)
}
  \item{min.length}{
Optionally exclude fibers below a certain fiber length (e.g., based on anatomical measurements). If resolution is given, then in those units, otherwise in number of voxels.
}
  \item{length.out}{
Number of line segments used in the calculation of fiber length
}
  \item{df}{
Degrees of freedom passed to splines::ns in the smoothing of muscle fibers before calculation. df = 1 produces straight fibers, while values > 1 produce increasingly curved fibers
}
}
\value{
\item{quality}{grayscale sd/fiber length, low values are considered of higher quality}
\item{grey.values}{A list providing the grayscale values for each smoothed fibers}
\item{problem.fibers}{The location of fibers in the original list object that have atypically poor quality and should be excluded from further analyses}
}
\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030.
Puffel, F. Pouget, A., Liu, X., Zuber, M., van de Kamp, T., Roces, F., and Labonte, D., 2021. \emph{Journal of the Royal Society Interface} 18: 20210424
}
\author{
J. Arbour
}
\note{
Also produces a plot showing the distribution of quality values, and numbered bars for outliers.
}
\seealso{
\code{\link{fuse.fibers}},\code{\link{check.overlap}},
}
\examples{
\donttest{

olddir<-getwd()

#### this downloads the ant dataset image stack
#### if you have it already downloaded you can navigate to that folder
setwd(tempdir())
download.file(url=
"https://github.com/jessica-arbour/Ant-Muscle-Image-Stack/raw/main/Ant_data.zip",
destfile="antdata.zip")

unzip("antdata.zip")
setwd(paste0(getwd(),"/Ant data"))
####

data(ant.raw)
images<-list.files(pattern=".png")

qc<-quality.check(ant.raw[21:50],images, res=0.000673107, df=1)


setwd(olddir)
}

}

\name{color.scale}
\alias{color.scale}
\title{
A simple wrapper to create colors for a continuous variable}
\description{
For any continuous variable like fiber length, produces a vector that can be used in plotting functions for the \emph{col} argument. Uses colorRampPalette.
}
\usage{
color.scale(fl, cols, min.fl=NULL, max.fl=NULL)
}
\arguments{
  \item{fl}{
A numeric vector containing a measurement for each fiber in a fiber list (length, curvature or angle).
}
  \item{cols}{
A character vector for determining the color gradient, in order from small to large values. Alternatively "viridis", "rainbow", "inferno" and "turbo" will supply these preset color gradients.
}
  \item{min.fl}{
A imposed lowest value of fl for the color scale. Can be used to make color legends equivalent across different plots.
}
\item{max.fl}{
A imposed highest value of fl for the color scale. Can be used to make color legends equivalent across different plots.
}
}
\value{
A vector with color values for each fiber}

\seealso{
\code{\link{muscle.plot.stl}}
}
\examples{
\donttest{
data(ant.final)
fl<-fiber.lengths(ant.final, res = 0.000673107, df=1)

cols<-color.scale(fl, c("blue", "green"))
muscle.plot.stl(ant.final, res = 0.000673107, cols = cols, mirror.axis = TRUE, df=1)
}
}

\name{crop.stack}
\alias{crop.stack}
\title{
Automated imaging cropping
}
\description{
Automatically crops a stack of png image files to the minimum bounds of non-black values. Or if bounds are supplied can be cropped to the exact size of another stack of images.
}
\usage{
crop.stack(images, bounds = NULL, save.images=FALSE)
}
\arguments{
  \item{images}{
A vector with file names for the image files, can be created with list.files.
}
  \item{bounds}{
An optional vector with four values, given as c(xlim, xmax, ylim, ymax). These are printed at the end of the function when cropping is performed automatically.
}
  \item{save.images}{
When TRUE, images are saved to the current directory using imager::save.image.
}
}
\value{
The bounds to be used for cropping. Optionally crops and saves images to working folder.
}
\references{
Arbour, J. 2023. GoodFibes: an R package for the detection of muscle fibers from diceCT scans. \emph{Integrative Organismal Biology} 5(1): obad030.}
\author{
J. Arbour}
\seealso{
\code{\link{equalize.stack}}
}
\examples{

\donttest{
olddir<-getwd()

#### this downloads the ant dataset image stack
#### if you have it already downloaded you can navigate to that folder
setwd(tempdir())
download.file(url=
"https://github.com/jessica-arbour/Ant-Muscle-Image-Stack/raw/main/Ant_data.zip",
destfile="antdata.zip")

unzip("antdata.zip")
setwd(paste0(getwd(),"/Ant data"))
####

images<-list.files(pattern=".png")

crop.stack(images)

setwd(olddir)
}



}
